# Sistema de Control de Temperatura
El Sistema de Control de Temperatura es un diseño basado en la recopilación de datos de la temperatura de dos transistores a partir de sensores que permiten medir esta variable y utilizando Arduino UNO como microcontrolador. Con la ayuda de pasta térmica, la energía del transistor al calentarse es transferida por conducción y convección al sensor de temperatura. Además, este proyecto dispone de una interfaz desarrollada en Python para visualizar la información y monitorear el estado de los dispositivos.


## Funcionamiento Básico
La información acerca de la temperatura registrada por el termistor es reunida por el microcontrolador para posteriormente ser enviada a través de comunicación serial a una computadora; mediante el código desarrollado y cargado al Arduino UNO, el monitor serial del software Arduino recibe estos datos, los cuales inmediatamente son expedidos al programa diseñado en Python para mostrar en una interfaz los resultados obtenidos en tiempo real tanto de manera numérica como gráfica, además de permitir controlar el encendido y apagado de los transistores.

## Programas Requeridos
En primer lugar se tiene que disponer del software de *Arduino* para poder comunicarse con el microcontrolador, cargar el programa en el mismo y recibir la información en el monitor serial. Para la descarga e instalación se **recomienda** seguir los pasos especificados en los siguientes enlaces:

### Para Windows:
* [Como instalar Arduino en Windows](https://arduino.cl/como-instalar-arduino-en-windows/).

### Para Ubuntu:
* [Cómo instalar Arduino IDE en las últimas versiones de Ubuntu](https://ubunlog.com/arduino-ide-en-ubuntu/).

De igual manera, se requiere del IDE de python para poder ejecutar el código de la interfaz que mostrará la información enviada a través de Arduino. Para la descarga e instalación se **recomienda** seguir los pasos especificados en los siguientes enlaces:

### Para Windows:
* [Download the latest version for Windows](https://www.python.org/downloads/).

* [Download Spyder for Windows](https://www.spyder-ide.org).

### Para Ubuntu:
* [IDLE Python, un entorno integrado de desarrollo para el aprendizaje](https://ubunlog.com/idle-python-entorno-desarrollo-aprendizaje/).

*Nota: es importante mencionar que los anteriores tutoriales de las páginas web recomendadas no son de nuestra autoría.*

##  Cargar y ejecutar los Programas
Primeramente se debe cargar el programa a la tarjeta de *Arduino UNO*, para ello hay que seguir las siguientes instrucciones:

1. **Conectar la tarjeta Arduino.**

La placa de desarrollo debe de ser conectada a la computadora al puerto USB. El LED de encendio de la placa debería de iluminarse.

2. **Ejecutar el IDE de Arduino**

Ir al escritorio o buscar la aplicación de Arduino.

3. **Abrir el archivo correspondiente**

Para ello únicamente hay que ir al menú *Archivo > Abrir > Buscar el archivo lecturaArduino*

4. **Cargar el programa en la tarjeta**

Es necesario indicar el dispositivo con el que se está trabajando, en *Herramientas > Placas > Seleccionar Arduino Uno*

También el puerto en el que se encuentra conectada la placa, en *Herramientas > Puerto > Seleccionar el puerto adecuado*

Ahora simplemente hay que dar click en *subir*


A continuación, ya es posible ejecutar el código de Python para disponer de la interfaz para entablar la comunicación y comenzar a recibir los datos de Arduino.

### Instalación de librerías de Python
Es altamente probable que, la primera vez que se ejecute el programa en el sistema operativo del usuario, se muestren ciertos errores debido a la ausencia de algunas librería que se importan en el programa necesarias para la ejecución del mismo y sus funciones. Por lo que, si este es el caso deben instalarse todos los módulos faltantes; antes es preciso verificar algunas cuestiones como:

* **Versión de Python del sistema**

Para conocer cuál es la versión de python que tenemos instalada solo hace falta escribir en la terminal la siguiente instrucción:

> python --version

Es probable que sea alguna de las versiones de python3 (no importa cual), si no es así es recomendable llevar a cabo esta actualización puesto que algunas librerías suelen no ser compatibles entre la versión 2 y 3 de Python.

* **Orden pip instalada**

Una vez realizado esto, hay que comprobar también si se encuentra instalado el paquete pip, este es un sistema de gestión de paquetes utilizado para instalar y administrar paquetes de software escritos en Python. Para ello escribimos en la terminal:

> pip list

Si el resultado es un error, advertencia o mensaje de que no se ha encontrado la orden especificada, hay que ejecutar en la terminal la instrucción y esperar a que finalice la instalación:

> sudo apt install python3-pip

Si ya se dispone de este paquete, ya se puede comenzar con la instalación de las librerías.

Algunas de los posibles errores emitidos por Python al ejecutar el programa acerca de los  módulos ausentes son:

* **No module named 'matplotlib'**

Para su instalación se ejecutar la siguiente línea en la terminal:

> pip install matplotlib

* **No module named 'serial'**

Para su instalación se ejecutar la siguiente línea en la terminal:

> pip install serial

* **No module named 'Tkinter'**

Para su instalación se ejecutar la siguiente línea en la terminal:

> pip install tkinter

* **cannot import name 'imageTk' from 'PIL'**

Este error es un poco distinto y suele ser por problemas de compatibilidad con el paquete pillow, se puede solucionar ejecutando la siguiente instrucción en la terminal:

> sudo apt-get install python3-pil python3-pil.imagetk

*Notas: es posible que al usar otro IDE de Python no se requiera solucionar estos errores por que los módulos ya vienen instalados con el programa; también puede que, si ocurren estos errores, el procedimiento de instalación de los módulos sea diferente al trabajar en un entorno distinto; además, si es el caso de que falta algún otro módulo de librería solo hay que ejecutar la instalación del mismo con la instrucción "pip install"; información adicional sobre otro tipo de errores no mencionados aquí se puede encontrar en diversas páginas web*  

## Código en Arduino
A continuación, se presenta una breve explicación acerca del funcionamiento del programa diseñado en el IDE de Arduino:

En primer lugar, se establecen los puertos a utilizar del microcontrolador, para ello se definen 2 constantes correspondientes a la entrada de datos analógicos (pines A0 Y A1) a partir de los sensores de temperatura y otras 2 para las salidas que encenderán a los transistores (pines 6 y 7); además de ello, se declaran las variables que guardan los datos de las lecturas recibidas y las que almacenan el valor convertido de la temperatura en grado centígrados, por último una variable que indica el estado proporcionado desde Python para encender o apagar los transistores:

```
#define in1 A0
#define in2 A1
#define out1 6
#define out2 7
float temp1, lectura1, temp2, lectura2;
String estado;
```

Ahora, en la función *setup()* se indica el uso del monitor serial a una velocidad de comunicación de 9600 baudios, además de el modo de funcionamiento de los pines de entrada y salida:

```
Serial.begin(9600);
pinMode(in1, INPUT);
pinMode(in2, INPUT);
pinMode(out1, OUTPUT);
pinMode(out2, OUTPUT);
```

En el ciclo prrincipal *loop()*, primeramente se establecen las instrucciones para la lectura de datos. las variables de lectura almacenan un valor analógico que consta de valor entre 0 a 1023; posterior a ello se utiliza una ecuación donde la parte que corresponde a **(lecturax * 5000 / 1024)** convierte la medición realizada a voltaje entre 0 a 5000mV; ahora, el fabricante del sensor TMP36 indica que la salida de tensión será de 10 mV (mili voltios) por cada grado de temperatura, por lo que el resultado del voltaje se divide entre 10 para realizar la conversión a temperatura; se imprimen los resultados en el monitor serial para poder leerlos desde Python:

```
lectura1 = analogRead(in1);
lectura2 = analogRead(in2);
temp1 = (lectura1 * 5000 / 1024 / 10) - 33;
temp2 = (lectura2 * 5000 / 1024 / 10) - 37;
Serial.println(temp1);
Serial.println(temp2);
```

Finalmente, se codifican las líneas para cambiar el estado del transistor dependiendo de la información recibida desde Python en el monitor serial, por lo que antes que nada se debe de verificar los datos disponibles en este, para luego realizar la lectura de la variable en este monitor y enviarla a la placa de Arduino para encender o apagar el transistor indicado: 

```
if(Serial.available()>0){
    estado = Serial.readString();
    if(estado == "on1"){
      digitalWrite(out1, HIGH);
    }
    else if(estado == "off1"){
      digitalWrite(out1, LOW);
    }
    else if(estado == "on2"){
      digitalWrite(out2, HIGH);
    }
    else if(estado == "off2"){
      digitalWrite(out2, LOW);
    }
}
```

Se establece también un retardo de 100 milisegundos para repetir el ciclo.

## Código en Python


![](http://gmarxcc.com:8088/MSP430/GUI-Heater-System/raw/branch/master/Esquematico%20Conexiones/Esquematico_Sistema_de_Calentamiento.png)

